@@ -9,9 +9,27 @@ from kea2.u2Driver import U2Driver
 class Omni_Notes_Sample(unittest.TestCase):
 
     def setUp(self):
-        self.d = u2.connect()
+        self.d = u2.connect() 
+    
+    @prob(0.2)
+    @precondition(
+        lambda self: self.d(description="Navigate up").exists
+    )
+    def test_goBack(self):
+        print("Navigate back")
+        self.d(description="Navigate up").click()
+        sleep(0.5)
+    
+    @prob(0.2)
+    @precondition(
+        lambda self: self.d(description="drawer closed").exists
+    )
+    def test_openDrawer(self):
+        print("Open drawer")
+        self.d(description="drawer closed").click()
+        sleep(0.5)
 
-    @prob(0.7)  # The probability of executing the function when precondition is satisfied.
+    @prob(0.5)  # The probability of executing the function when precondition is satisfied.
     @precondition(
         lambda self: self.d(text="Omni Notes Alpha").exists
         and self.d(text="Settings").exists
@@ -45,6 +63,7 @@ class Omni_Notes_Sample(unittest.TestCase):
         assertion:
             The search input box is still being opened
         """
+        print("rotate the device")
         self.d.set_orientation("l")
         sleep(2)
         self.d.set_orientation("n")
@@ -52,7 +71,7 @@ class Omni_Notes_Sample(unittest.TestCase):
         assert self.d(resourceId="it.feio.android.omninotes.alpha:id/search_src_text").exists()
 
 
-URL = "https://raw.githubusercontent.com/ecnusse/Kea2/refs/heads/dev_test_hidden_algorithm/omninotes.apk"
+URL = "https://github.com/federicoiosue/Omni-Notes/releases/download/6.2.0_alpha/OmniNotes-alphaRelease-6.2.0.apk"
 PACKAGE_NAME = "it.feio.android.omninotes.alpha"
 FILE_NAME = "omninotes.apk"
 
