@@ -375,19 +375,34 @@ class BugReportGenerator:
                 minutes, seconds = divmod(remainder, 60)
                 data["total_testing_time"] = f"{hours:02d}:{minutes:02d}:{seconds:02d}"
 
-        # Calculate bug count directly from result data
+        # Enrich property statistics with derived metrics and calculate bug count
+        enriched_property_stats = {}
         for property_name, test_result in self.test_result.items():
             # Check if failed or error
-            if test_result["fail"] > 0 or test_result["error"] > 0:
+            if test_result.get("fail", 0) > 0 or test_result.get("error", 0) > 0:
                 data["bugs_found"] += 1
 
-        # Store the raw result data for direct use in HTML template
-        data["property_stats"] = self.test_result
+            executed_count = test_result.get("executed", 0)
+            fail_count = test_result.get("fail", 0)
+            error_count = test_result.get("error", 0)
+            pass_count = max(executed_count - fail_count - error_count, 0)
+
+            enriched_property_stats[property_name] = {
+                **test_result,
+                "pass_count": pass_count
+            }
+
+        # Store the enriched result data for direct use in HTML template
+        data["property_stats"] = enriched_property_stats
 
         # Calculate properties statistics
         data["all_properties_count"] = len(self.test_result)
         data["executed_properties_count"] = sum(1 for result in self.test_result.values() if result.get("executed", 0) > 0)
 
+        # Calculate detailed property statistics for table headers
+        property_stats_summary = self._calculate_property_stats_summary(enriched_property_stats)
+        data["property_stats_summary"] = property_stats_summary
+
         # Process coverage data
         data["coverage_trend"] = self.cov_trend
 
@@ -411,6 +426,11 @@ class BugReportGenerator:
 
         # Load crash and ANR events from crash-dump.log
         crash_events, anr_events = self._load_crash_dump_data()
+
+        # Add screenshot ID information to crash and ANR events
+        self._add_screenshot_ids_to_events(crash_events)
+        self._add_screenshot_ids_to_events(anr_events)
+
         data["crash_events"] = crash_events
         data["anr_events"] = anr_events
 
@@ -456,10 +476,14 @@ class BugReportGenerator:
         """
         screenshot_path: Path = self.data_path.screenshots_dir / screenshot_name
         if not screenshot_path.exists():
-            logger.error(f"Screenshot file {screenshot_path} not exists.")
+            logger.debug(f"Screenshot file {screenshot_path} not exists.")
             return False
 
-        img = Image.open(screenshot_path).convert("RGB")
+        try:
+            img = Image.open(screenshot_path).convert("RGB")
+        except OSError as e:
+            logger.debug(f"Error opening image {screenshot_path}: {e}")
+            return False
         draw = ImageDraw.Draw(img)
         line_width = 5
 
@@ -534,7 +558,7 @@ class BugReportGenerator:
         timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
 
         # Ensure coverage_trend has data
-        if not data["coverage_trend"]:
+        if not data.get("coverage_trend"):
             logger.warning("No coverage trend data")
             # Use the same field names as in coverage.log file
             data["coverage_trend"] = [{"stepsCount": 0, "coverage": 0, "testedActivitiesCount": 0}]
@@ -567,7 +591,8 @@ class BugReportGenerator:
             'property_execution_data': json.dumps(data["property_execution_trend"]),
             'activity_count_history': data["activity_count_history"],
             'crash_events': data["crash_events"],
-            'anr_events': data["anr_events"]
+            'anr_events': data["anr_events"],
+            'property_stats_summary': data["property_stats_summary"]
         }
 
         # Check if template exists, if not create it
@@ -607,6 +632,13 @@ class BugReportGenerator:
             caption = f"{prop_name}: {state}" if prop_name else f"{state}"
 
         screenshot_name = step_data["Screenshot"]
+
+        # Check if the screenshot file actually exists
+        screenshot_file_path = self.data_path.screenshots_dir / screenshot_name
+        if not screenshot_file_path.exists():
+            # Skip adding this screenshot if the file doesn't exist
+            return
+
         # Use relative path string instead of Path object
         relative_screenshot_path = f"output_{self.log_timestamp}/screenshots/{screenshot_name}"
 
@@ -822,6 +854,46 @@ class BugReportGenerator:
         
         return property_execution_trend
 
+    def _calculate_property_stats_summary(self, test_result: TestResult) -> Dict[str, int]:
+        """
+        Calculate summary statistics for property checking table headers
+
+        Args:
+            test_result: Test result data containing property statistics
+
+        Returns:
+            Dict: Summary statistics for each column
+        """
+        stats_summary = {
+            "total_properties": 0,
+            "total_precond_satisfied": 0,
+            "total_executed": 0,
+            "total_passes": 0,
+            "total_fails": 0,
+            "total_errors": 0,
+            "properties_with_errors": 0
+        }
+
+        for property_name, result in test_result.items():
+            executed_count = result.get("executed", result.get("executed_total", 0))
+            fail_count = result.get("fail", 0)
+            error_count = result.get("error", 0)
+            pass_count = result.get("pass_count",
+                                    max(executed_count - fail_count - error_count, 0))
+
+            stats_summary["total_properties"] += 1
+            stats_summary["total_precond_satisfied"] += result.get("precond_satisfied", 0)
+            stats_summary["total_executed"] += executed_count
+            stats_summary["total_passes"] += pass_count
+            stats_summary["total_fails"] += fail_count
+            stats_summary["total_errors"] += error_count
+
+            # Count properties that have errors or fails
+            if fail_count > 0 or error_count > 0:
+                stats_summary["properties_with_errors"] += 1
+
+        return stats_summary
+
     def _load_crash_dump_data(self) -> Tuple[List[Dict], List[Dict]]:
         """
         Load crash and ANR events from crash-dump.log file
@@ -840,11 +912,11 @@ class BugReportGenerator:
             with open(self.data_path.crash_dump_log, "r", encoding="utf-8") as f:
                 content = f.read()
 
-            # Parse crash events
-            crash_events = self._parse_crash_events(content)
+            # Parse crash events with screenshot mapping
+            crash_events = self._parse_crash_events_with_screenshots(content)
 
-            # Parse ANR events
-            anr_events = self._parse_anr_events(content)
+            # Parse ANR events with screenshot mapping
+            anr_events = self._parse_anr_events_with_screenshots(content)
 
             logger.debug(f"Found {len(crash_events)} crash events and {len(anr_events)} ANR events")
 
@@ -852,26 +924,29 @@ class BugReportGenerator:
 
         except Exception as e:
             logger.error(f"Error reading crash dump file: {e}")
+            return crash_events, anr_events
 
-
-    def _parse_crash_events(self, content: str) -> List[Dict]:
+    def _parse_crash_events_with_screenshots(self, content: str) -> List[Dict]:
         """
-        Parse crash events from crash-dump.log content
+        Parse crash events from crash-dump.log content with screenshot mapping
 
         Args:
             content: Content of crash-dump.log file
 
         Returns:
-            List[Dict]: List of crash event dictionaries
+            List[Dict]: List of crash event dictionaries with screenshot information
         """
         crash_events = []
 
-        # Pattern to match crash blocks
-        crash_pattern = r'(\d{14})\ncrash:\n(.*?)\n// crash end'
+        # Pattern to match crash blocks with optional screenshot information
+        # Look for StepsCount and CrashScreen before the timestamp
+        crash_pattern = r'(?:StepsCount:\s*(\d+)\s*\nCrashScreen:\s*([^\n]+)\s*\n)?(\d{14})\ncrash:\n(.*?)\n// crash end'
 
         for match in re.finditer(crash_pattern, content, re.DOTALL):
-            timestamp_str = match.group(1)
-            crash_content = match.group(2)
+            steps_count = match.group(1)
+            crash_screen = match.group(2)
+            timestamp_str = match.group(3)
+            crash_content = match.group(4)
 
             # Parse timestamp (format: YYYYMMDDHHMMSS)
             try:
@@ -887,31 +962,35 @@ class BugReportGenerator:
                 "time": formatted_time,
                 "exception_type": crash_info.get("exception_type", "Unknown"),
                 "process": crash_info.get("process", "Unknown"),
-                "stack_trace": crash_info.get("stack_trace", "")
+                "stack_trace": crash_info.get("stack_trace", ""),
+                "steps_count": steps_count,
+                "crash_screen": crash_screen.strip() if crash_screen else None
             }
 
             crash_events.append(crash_event)
 
         return crash_events
 
-    def _parse_anr_events(self, content: str) -> List[Dict]:
+    def _parse_anr_events_with_screenshots(self, content: str) -> List[Dict]:
         """
-        Parse ANR events from crash-dump.log content
+        Parse ANR events from crash-dump.log content with screenshot mapping
 
         Args:
             content: Content of crash-dump.log file
 
         Returns:
-            List[Dict]: List of ANR event dictionaries
+            List[Dict]: List of ANR event dictionaries with screenshot information
         """
         anr_events = []
 
-        # Pattern to match ANR blocks
-        anr_pattern = r'(\d{14})\nanr:\n(.*?)\nanr end'
+        # Pattern to match ANR blocks with optional screenshot information
+        anr_pattern = r'(?:StepsCount:\s*(\d+)\s*\nCrashScreen:\s*([^\n]+)\s*\n)?(\d{14})\nanr:\n(.*?)\nanr end'
 
         for match in re.finditer(anr_pattern, content, re.DOTALL):
-            timestamp_str = match.group(1)
-            anr_content = match.group(2)
+            steps_count = match.group(1)
+            crash_screen = match.group(2)
+            timestamp_str = match.group(3)
+            anr_content = match.group(4)
 
             # Parse timestamp (format: YYYYMMDDHHMMSS)
             try:
@@ -927,13 +1006,51 @@ class BugReportGenerator:
                 "time": formatted_time,
                 "reason": anr_info.get("reason", "Unknown"),
                 "process": anr_info.get("process", "Unknown"),
-                "trace": anr_info.get("trace", "")
+                "trace": anr_info.get("trace", ""),
+                "steps_count": steps_count,
+                "crash_screen": crash_screen.strip() if crash_screen else None
             }
 
             anr_events.append(anr_event)
 
         return anr_events
 
+    def _find_screenshot_id_by_filename(self, screenshot_filename: str) -> str:
+        """
+        Find screenshot ID by filename in the screenshots list
+
+        Args:
+            screenshot_filename: Name of the screenshot file
+
+        Returns:
+            str: Screenshot ID if found, empty string otherwise
+        """
+        if not screenshot_filename:
+            return ""
+
+        for screenshot in self.screenshots:
+            # Extract filename from path
+            screenshot_path = screenshot.get('path', '')
+            if screenshot_path.endswith(screenshot_filename):
+                return str(screenshot.get('id', ''))
+
+        return ""
+
+    def _add_screenshot_ids_to_events(self, events: List[Dict]):
+        """
+        Add screenshot ID information to crash/ANR events
+
+        Args:
+            events: List of crash or ANR event dictionaries
+        """
+        for event in events:
+            crash_screen = event.get('crash_screen')
+            if crash_screen:
+                screenshot_id = self._find_screenshot_id_by_filename(crash_screen)
+                event['screenshot_id'] = screenshot_id
+            else:
+                event['screenshot_id'] = ""
+
     def _extract_crash_info(self, crash_content: str) -> Dict:
         """
         Extract crash information from crash content
@@ -1072,8 +1189,8 @@ class BugReportGenerator:
 if __name__ == "__main__":
     print("Generating bug report")
     # OUTPUT_PATH = "<Your output path>"
-    OUTPUT_PATH = "P:/Python/Kea2/output/res_2025072011_5048015228"
+    OUTPUT_PATH = "/Users/drifter327/Code/Kea2/output/res_2025090122_1216279438"
 
     report_generator = BugReportGenerator()
     report_path = report_generator.generate_report(OUTPUT_PATH)
-    print(f"bug report generated: {report_path}")
\ No newline at end of file
+    print(f"bug report generated: {report_path}")
