@@ -2,33 +2,12 @@ import logging
 import os
 from pathlib import Path
 import traceback
-from typing import TYPE_CHECKING
+from typing import Callable, Dict, Optional
 
 import time
 from functools import wraps
-if TYPE_CHECKING:
-    from .keaUtils import Options
 
 
-def getLogger(name: str) -> logging.Logger:
-    logger = logging.getLogger(name)
-
-    def enable_pretty_logging():
-        if not logger.handlers:
-            # Configure handler
-            handler = logging.StreamHandler()
-            handler.flush = lambda: handler.stream.flush()  # 确保每次都flush
-            formatter = logging.Formatter('[%(levelname)1s][%(asctime)s %(module)s:%(lineno)d pid:%(process)d] %(message)s')
-            handler.setFormatter(formatter)
-            logger.addHandler(handler)
-            logger.propagate = False
-
-    enable_pretty_logging()
-    return logger
-
-
-logger = getLogger(__name__)
-
 
 def singleton(cls):
     _instance = {}
@@ -39,6 +18,51 @@ def singleton(cls):
         return _instance[cls]
     return inner
 
+
+
+class LoggingLevel:
+    level = logging.INFO
+    _instance: Optional["LoggingLevel"] = None  # 单例缓存
+
+    def __new__(cls):
+        if cls._instance is None:
+            cls._instance = super().__new__(cls)
+        return cls._instance
+
+    @classmethod
+    def set_level(cls, level: int):
+        cls.level = level
+
+
+class DynamicLevelFilter(logging.Filter):
+    def filter(self, record: logging.LogRecord) -> bool:
+        return record.levelno >= LoggingLevel.level
+
+
+def getLogger(name: str) -> logging.Logger:
+    logger = logging.getLogger(name)
+
+    def enable_pretty_logging():
+        if not logger.handlers:
+            # Configure handler
+            handler = logging.StreamHandler()
+            handler.flush = lambda: handler.stream.flush()
+            handler.setFormatter(logging.Formatter('[%(levelname)1s][%(asctime)s %(module)s:%(lineno)d pid:%(process)d] %(message)s'))
+            handler.setLevel(logging.NOTSET)
+            handler.addFilter(DynamicLevelFilter())
+            logger.addHandler(handler)
+            logger.setLevel(logging.DEBUG)
+            logger.propagate = False
+
+    enable_pretty_logging()
+    return logger
+
+
+logger = getLogger(__name__)
+
+
+
+
 @singleton
 class TimeStamp:
     time_stamp = None
@@ -109,4 +133,26 @@ def catchException(log_info: str):
                 tb = traceback.format_exception(type(e), e, e.__traceback__.tb_next)
                 print(''.join(tb), end='', flush=True)
         return wrapper
-    return accept
\ No newline at end of file
+    return accept
+
+
+def loadFuncsFromFile(file_path: str) -> Dict[str, Callable]:
+    if not os.path.exists(file_path):
+        raise FileNotFoundError(f"{file_path} not found.")
+
+    def __get_module():
+        import importlib.util
+        module_name = Path(file_path).stem
+        spec = importlib.util.spec_from_file_location(module_name, file_path)
+        mod = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(mod)
+        return mod
+
+    mod = __get_module()
+
+    import inspect
+    funcs = dict()
+    for func_name, func in inspect.getmembers(mod, inspect.isfunction):
+        funcs[func_name] = func
+
+    return funcs
\ No newline at end of file
